@@ -4,6 +4,10 @@
 
 set -e
 
+# Run in most basic of locals to avoid performance overhead (and risk of bugs)
+# involved in localization, encoding issues etc.  We only do ASCII here.
+export LC_ALL=C
+
 ver=""
 for amv in "1.6" "1.7" "1.8" "1.9" ; do
 	if which "automake-$amv" >/dev/null 2>&1; then
@@ -34,7 +38,7 @@ automake${ver} --verbose --add-missing --copy
 autoconf
 
 conf_flags="--enable-maintainer-mode $CONFIG_ARGS"
-if test x$NOCONFIGURE = x; then
+if test -z "$NOCONFIGURE" ; then
 	echo Running $srcdir/configure $conf_flags "$@" ...
 	./configure $conf_flags "$@" \
 	&& echo Now type \`make\' to compile $PKG_NAME || exit 1
