@@ -32,11 +32,42 @@
 #include <windows.h>
 #endif
 
+#include "libpq-fe.h"
+
 #include "pqxx/util"
 
 using namespace PGSTD;
-using namespace pqxx::internal::pq;
 
+namespace
+{
+// It turns out that NaNs are pretty hard to do portably.  If the appropriate
+// C++ traits functions are not available, C99 defines some functions that can
+// do the same work.  But if none of those are available, we need to resort to
+// compile-time "0/0" expressions.  Most compilers won't crash while compiling
+// those anymore, but there may be unneeded warnings--which are almost as bad.
+template<typename T> void set_to_NaN(T &);
+
+#if defined(PQXX_HAVE_QUIET_NAN)
+template<typename T> inline void set_to_NaN(T &t)
+	{ t = numeric_limits<T>::quiet_NaN(); }
+#elif defined(PQXX_HAVE_NAN)
+template<> inline void set_to_NaN(float &t) { t = fnan(""); }
+template<> inline void set_to_NaN(double &t) { t = nan(""); }
+#ifdef PQXX_HAVE_LONG_DOUBLE
+template<> inline void set_to_NaN(long double &t) { t = lnan(""); }
+#endif
+#else
+const float nan_f(0.0/0.0);
+template<> inline void set_to_NaN(float &t) { t = nan_f; }
+const double nan_d(0.0/0.0);
+template<> inline void set_to_NaN(double &t) { t = nan_d; }
+#ifdef PQXX_HAVE_LONG_DOUBLE
+const long double nan_ld(0.0/0.0);
+template<> inline void set_to_NaN(long double &t) { t = nan_ld; }
+#endif
+
+#endif
+}
 
 namespace pqxx
 {
@@ -139,12 +170,13 @@ template<typename T> inline void from_string_unsigned(const char Str[], T &Obj)
 template<typename T> inline void from_string_float(const char Str[], T &Obj)
 {
   bool ok = false;
-  T result = numeric_limits<T>::quiet_NaN();
+  T result;
 
   if (tolower(Str[0])=='n')
   {
     // Accept "NaN," "nan," etc.
     ok = (tolower(Str[1])=='a' && tolower(Str[2])=='n' && !Str[3]);
+    set_to_NaN(result);
   }
   else
   {
@@ -540,6 +572,20 @@ string pqxx::internal::Quote_charptr(const char Obj[], bool EmptyIsNull)
 }
 
 
+template<>
+void pqxx::internal::PQAlloc<pqxx::internal::pq::PGresult>::freemem() throw ()
+{
+  PQclear(m_Obj);
+}
+
+
+template<>
+void pqxx::internal::PQAlloc<pqxx::internal::pq::PGnotify>::freemem() throw ()
+{
+  freenotif(m_Obj);
+}
+
+
 string pqxx::internal::namedclass::description() const
 {
   try
@@ -611,6 +657,8 @@ void pqxx::internal::freenotif(PGnotify *p)
 
 void pqxx::internal::sleep_seconds(int s)
 {
+  if (s <= 0) return;
+
 #if defined(PQXX_HAVE_SLEEP)
   // Use POSIX.1 sleep() if available
   sleep(s);
@@ -624,8 +672,17 @@ void pqxx::internal::sleep_seconds(int s)
   struct timeval timeout;
   timeout.tv_sec = s;
   timeout.tv_usec = 0;
-  if (select(0, &F, &F, &F, &timeout) == -1)
-    throw runtime_error(strerror(errno));
+  if (select(0, &F, &F, &F, &timeout) == -1) switch (errno)
+  {
+  case EINVAL:	// Invalid timeout
+	throw out_of_range("Invalid timeout value: " + to_string(s));
+  case EINTR:	// Interrupted by signal
+	break;
+  case ENOMEM:	// Out of memory
+	throw bad_alloc();
+  default:
+    throw logic_error("libpqxx internal error in select()");
+  }
 #endif
 }
 
